@@ -1,12 +1,11 @@
 ENCODING = "utf-8"
-SIZE_1GB = 1_073_741_824
-
+SIZE_1GB = 1_073_741_824  # 1GB
 LMDB_MAX_KEY = 511
-LMDB_MAP_SIZE = 10_737_418_240  # 10GB
+LMDB_MAP_SIZE = SIZE_1GB
 LMDB_BUFF_LIMIT = SIZE_1GB
 
 
-class ToBytesType:
+class ToBytes:
     OBJ = 0
     INT_NUMPY = 1
     INT_BITMAP = 2
